package org.springblade.sms.component;

import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.dto.DefineGroup;
import org.springblade.sms.enums.GroupType;
import org.springblade.sms.enums.SmsPushStatus;
import org.springblade.sms.kafka.PushTaskProducer;
import org.springblade.sms.kafka.SmsDataKafka;
import org.springblade.sms.service.AdsMarketTaskInfoService;
import org.springblade.sms.service.DefineGroupService;
import org.springblade.sms.service.clickhouse.CubeGroupService;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.service.UserGroupCalculateService;
import org.springblade.useranalysis.service.UserGroupService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import cn.hutool.core.util.PhoneUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 邮件或短信任务推送处理服务
 *
 * @Author：william
 * @Date：2022/5/26
 */
@Slf4j
@Service
public class MarketTaskPushService {
	/**
	 * 分页数量
	 */
	private final static int PHONE_PAGE_SIZE = 1000;

	@Autowired
	private UserGroupService userGroupService;

	@Autowired
	private DefineGroupService defineGroupService;

	@Autowired
	private CubeGroupService cubeGroupService;

	@Autowired
	private UserGroupCalculateService userGroupCalculateService;

	@Autowired
	private PushTaskProducer pushTaskProducer;

	@Autowired
	private AdsMarketTaskInfoService taskService;

	private final int maxTimes = 5;

	@Autowired
	private SmsSender smsSender;

	@Async
	public void asyncPushSms(AdsMarketTaskInfo task, int curTime) {
		// windata测试后，打开该代码段
		if (!smsSender.isProd()) {
			return;
		}

		log.info("MarketTaskPushService:asyncPushSms start send sms");
		try {
			if (task.getTargetGroupNumber() >= 0) {
				GroupType groupType = GroupType.reslove(task.getGroupType());
				DefineGroup group = null;
				if (groupType == GroupType.custom_group) {
					// 自定义人群且为动态人群，则先判断当前人群是否完成
					group = defineGroupService.getById(task.getGroupId());
					if (group.getStatus() != 2) {
						// 人群未完成，两分钟后重试
						if (curTime >= maxTimes) {
							// 尝试五次，短信任务失败
							task.setErrorInfo("动态人群，每两分钟尝试游戏啊，5次后人群任务仍未完成");
							updateTaskStatus(task, SmsPushStatus.fail);
						} else {
							try {
								TimeUnit.MINUTES.sleep(2);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							asyncPushSms(task, curTime++);
						}
						return;
					}

					if (task.getTargetGroupNumber() == 0) {
						// 说明该动态人群还没有查询过总人数
						long count = group.getAmount();
						task.setTargetGroupNumber(count);
						taskService.updateById(task);
					}
				}

				// 下次查询偏移量
				int offset = 0;
				while (true) {
					List<String> mobileList = null;
					if (groupType == GroupType.tag_group) {
						// 标签人群
						UserGroupEntity userGroupEntity = userGroupService.getById(task.getGroupId());
						mobileList = userGroupCalculateService.queryUserIdList(userGroupEntity, offset,
								PHONE_PAGE_SIZE);
					} else if (groupType == GroupType.import_group) {
						// 导入人群
						mobileList = userGroupCalculateService.queryUserIdListFromImport(task.getGroupId(), offset,
								PHONE_PAGE_SIZE);
					} else {
						// 自定义人群
						mobileList = cubeGroupService.getPhoneList(group, offset, PHONE_PAGE_SIZE);
					}
					if (mobileList == null || mobileList.isEmpty()) {
						// 没有数据
						break;
					}
					for (String mobile : mobileList) {
						if (!PhoneUtil.isMobile(mobile)) {
							// 过滤非手机号
							continue;
						}
						pushTaskProducer.sms(new SmsDataKafka(mobile, task));
					}

					offset += mobileList.size();

					if (mobileList.size() < PHONE_PAGE_SIZE) {
						// 当前已经是最后一页
						break;
					}
				}

				updateTaskStatus(task, SmsPushStatus.finish);
			} else {
				log.warn("MarketTaskPushService:asyncPushSms 人群数量为0,不再短信推送.taskId=" + task.getTaskId());
			}
		} catch (Exception ex) {
			log.error(ex.getMessage(), ex);
		}
	}

//	@Async
//	public void asyncPushEmail(AdsMarketTaskInfo task) {
//		log.info("MarketTaskPushService:asyncPushEmail start send email");
//		try {
//			if (task.getTargetGroupNumber() >= 0) {
//				List<AdsTempContactInfo> list = adsTempContactInfoService.list();
//				for (AdsTempContactInfo user : list) {
//					String mobile = user.getMobile();
//					String email = user.getEmail();
//					if (StringUtils.isBlank(email) || StringUtils.isBlank(mobile)) {
//						continue;
//					}
//
//					pushTaskProducer.email(mobile, email, task);
//				}
//				updateTaskStatus(task, AdsMarketPushStatus.finish);
//			} else {
//				log.warn("MarketTaskPushService:asyncPushEmail 人群数量为0,不再邮件推送.taskId=" + task.getTaskId());
//			}
//		} catch (Exception e) {
//			log.error(e.getMessage(), e);
//		}
//	}

	/**
	 * 更新任务状态
	 */
	protected void updateTaskStatus(AdsMarketTaskInfo task, SmsPushStatus status) {
		AdsMarketTaskInfo entity = new AdsMarketTaskInfo();
		entity.setTaskId(task.getTaskId());
		entity.setPushStatus(status.getValue());
		taskService.updateById(entity);
	}
}
