package org.springblade.sms.component;

import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.dto.AdsMarketTaskInfoUrl;
import org.springblade.sms.service.AdsMarketTaskInfoUrlCodeGeneratorService;
import org.springblade.sms.service.AdsMarketTaskInfoUrlService;
import org.springblade.sms.util.Number62;
import org.springblade.useranalysis.common.utils.DesUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import lombok.extern.slf4j.Slf4j;

/**
 * 短链接服务
 * 
 * @author William
 * @date 2022年11月10日 上午11:10:11
 */
@Slf4j
@Service
public class ShortLinkService {

	@Value("${blade.shortUrl.domain:}")
	private String domain;

	@Autowired
	private AdsMarketTaskInfoUrlCodeGeneratorService codeService;

	@Autowired
	private AdsMarketTaskInfoUrlService urlService;

	@Autowired
	private BladeRedis redisService;

	/**
	 * 生成短连接
	 *
	 * @param mobile 手机号
	 * @param task 任务
	 * @return
	 */
	public String getShortLink(String mobile, AdsMarketTaskInfo task) {
		if (StrUtil.isNotBlank(task.getOriginUrl())) {
			String shortLink = getShortLink(task.getOriginUrl(), DesUtil.urlEncode(mobile, DesUtil.PASSWORD),
					task.getTaskId().toString(), task.getChannelId());
			return shortLink;
		}

		return null;
	}

	/**
	 * 生成短链接
	 *
	 * @param originUrl 带占位符的参数链接
	 * @param uid       用户id
	 * @param tid       任务id
	 * @param cid       渠道id
	 * @return 短链接全地址
	 */
	public String getShortLink(String originUrl, String uid, String tid, String cid) {
		if (StringUtils.isBlank(originUrl)) {
			return null;
		}
		String code = getCode();
		if (StringUtils.isBlank(code)) {
			return null;
		}

		String realUrl = originUrl.replaceAll("\\$\\{tid\\}", tid).replaceAll("\\$\\{uid\\}", uid)
				.replaceAll("\\$\\{cid\\}", cid);
		realUrl = realUrl.replace("&amp;", "&");
		AdsMarketTaskInfoUrl urlBean = new AdsMarketTaskInfoUrl();
		urlBean.setRealUrl(realUrl);
		urlBean.setCreateTime(new Date());
		urlBean.setCode(code);
		if (urlService.save(urlBean)) {
			redisService.setEx(urlBean.getCode(), realUrl, 60 * 60 * 6L);
			return domain + "/" + urlBean.getCode();
		}

		return null;
	}

	/**
	 * 生成短链接编码
	 */
	private String getCode() {
//		LambdaQueryWrapper<AdsMarketTaskInfoUrl> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfoUrl.class)
//				.orderByDesc(AdsMarketTaskInfoUrl::getUrlId).last("limit 0,1");
//		AdsMarketTaskInfoUrl data = urlService.getOne(wrapper);
//		Long codeNum = data != null ? Number62.decode(data.getCode()) + 1 : 1;
		Long codeNum = codeService.getCodeNum();
		String shortCode = Number62.encode(codeNum.longValue());
		log.info("ShortLinkService num is " + codeNum);
		log.info("ShortLinkService code is " + shortCode);
		return shortCode;
	}

	/**
	 * 根据短链接编码获取真实链接
	 *
	 * @param code 短链接编码
	 * @return 真实链接
	 */
	public String getRealUrl(String code) {
		String realUrl = null;
		// 从redis取真实链接
		Object value = redisService.get(code);
		if (value != null) {
			realUrl = value.toString();
		}

		if (StringUtils.isBlank(realUrl)) {
			// 从数据库取真实链接
			LambdaQueryWrapper<AdsMarketTaskInfoUrl> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfoUrl.class)
					.eq(AdsMarketTaskInfoUrl::getCode, code).last("limit 0,1");
			AdsMarketTaskInfoUrl infoUrl = urlService.getOne(wrapper);
			if (infoUrl != null) {
				realUrl = infoUrl.getRealUrl();
			}
		}

		return realUrl;
	}
}
