package org.springblade.sms.job;

import java.util.Date;
import java.util.List;

import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.component.MarketTaskPushService;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.SmsPushPattern;
import org.springblade.sms.enums.SmsPushStatus;
import org.springblade.sms.enums.TaskDraftStatus;
import org.springblade.sms.service.AdsMarketTaskInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 短信周期性发送的定时检查任务
 * 
 * @author William
 * @date 2022年11月15日 下午5:07:26
 */
@Slf4j
@Configuration
@EnableScheduling
public class CycleSendJob {
	@Autowired
	private AdsMarketTaskInfoService taskService;

	@Autowired
	private MarketTaskPushService pushService;

	/**
	 * 2分钟检查一次
	 */
	@Scheduled(cron = "0 0/2 * * * ?")
	private void loop() {
		findByDay();
		findByWeek();
		findByMonth();
	}

	/**
	 * 查询按天的
	 */
	private void findByDay() {
		try {
			Date now = new Date();
			String timeStr = DateUtil.format(now, "HHmmss");
			int timeValue = Integer.parseInt(timeStr);
			LambdaQueryWrapper<AdsMarketTaskInfo> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfo.class)
					.eq(AdsMarketTaskInfo::getPushPattern, SmsPushPattern.cycleTime.getValue())
					.eq(AdsMarketTaskInfo::getPushStatus, SmsPushStatus.waiting.getValue())
					.eq(AdsMarketTaskInfo::getCheckStatus, CheckStatus.ok.getValue())
					.eq(AdsMarketTaskInfo::getDraftStaus, TaskDraftStatus.not_draft.getValue())
					.ge(AdsMarketTaskInfo::getCycleStartTime, now)
					.le(AdsMarketTaskInfo::getCycleEndTime, now)
					.le(AdsMarketTaskInfo::getCycleSendTime, timeValue);
			List<AdsMarketTaskInfo> messageList = taskService.list(wrapper);
			// 更新推送状态为进行中【此处不使用批量更新，是考虑如果某个任务更新状态失败，无法识别】
			if (messageList != null && !messageList.isEmpty()) {
				log.info("MarketTaskPushService:found {} tasks", messageList.size());
				messageList.forEach(task -> {
					AdsMarketTaskInfo entity = new AdsMarketTaskInfo();
					entity.setTaskId(task.getTaskId());
					entity.setPushStatus(SmsPushStatus.doing.getValue());
					boolean ret = taskService.updateById(entity);
					if (ret) { // 更新成功，添加到推送队列
						pushService.asyncPushSms(task, 1);
					}
				});
			}
		} catch (Exception e) {
			log.info("CycleSendJob::findByDay exception", e.getMessage());
		}
	}

	/**
	 * 查询按周的
	 */
	private void findByWeek() {
		try {
			Date now = new Date();
			String timeStr = DateUtil.format(now, "HHmmss");
			int timeValue = Integer.parseInt(timeStr);
			int week = DateUtil.dayOfMonth(now);
			LambdaQueryWrapper<AdsMarketTaskInfo> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfo.class)
					.eq(AdsMarketTaskInfo::getPushPattern, SmsPushPattern.cycleTime.getValue())
					.eq(AdsMarketTaskInfo::getPushStatus, SmsPushStatus.waiting.getValue())
					.eq(AdsMarketTaskInfo::getCheckStatus, CheckStatus.ok.getValue())
					.eq(AdsMarketTaskInfo::getDraftStaus, TaskDraftStatus.not_draft.getValue())
					.ge(AdsMarketTaskInfo::getCycleStartTime, now)//
					.le(AdsMarketTaskInfo::getCycleEndTime, now)//
					.eq(AdsMarketTaskInfo::getCycleWeek, week)//
					.le(AdsMarketTaskInfo::getCycleSendTime, timeValue);
			List<AdsMarketTaskInfo> messageList = taskService.list(wrapper);
			// 更新推送状态为进行中【此处不使用批量更新，是考虑如果某个任务更新状态失败，无法识别】
			if (messageList != null && !messageList.isEmpty()) {
				log.info("MarketTaskPushService:found {} tasks", messageList.size());
				messageList.forEach(task -> {
					AdsMarketTaskInfo entity = new AdsMarketTaskInfo();
					entity.setTaskId(task.getTaskId());
					entity.setPushStatus(SmsPushStatus.doing.getValue());
					boolean ret = taskService.updateById(entity);
					if (ret) { // 更新成功，添加到推送队列
						pushService.asyncPushSms(task, 1);
					}
				});
			}
		} catch (Exception e) {
			log.info("CycleSendJob::findByWeek exception is {}", e.getMessage());
		}
	}

	/**
	 * 查询按月的
	 */
	private void findByMonth() {
		try {
			Date now = new Date();
			int day = DateUtil.dayOfWeek(now);
			String timeStr = DateUtil.format(now, "HHmmss");
			int timeValue = Integer.parseInt(timeStr);
			LambdaQueryWrapper<AdsMarketTaskInfo> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfo.class)
					.eq(AdsMarketTaskInfo::getPushPattern, SmsPushPattern.cycleTime.getValue())
					.eq(AdsMarketTaskInfo::getPushStatus, SmsPushStatus.waiting.getValue())
					.eq(AdsMarketTaskInfo::getCheckStatus, CheckStatus.ok.getValue())
					.eq(AdsMarketTaskInfo::getDraftStaus, TaskDraftStatus.not_draft.getValue())
					.ge(AdsMarketTaskInfo::getCycleStartTime, now)//
					.le(AdsMarketTaskInfo::getCycleEndTime, now)//
					.eq(AdsMarketTaskInfo::getCycleDayOfMonth, day)//
					.le(AdsMarketTaskInfo::getCycleSendTime, timeValue);
			List<AdsMarketTaskInfo> messageList = taskService.list(wrapper);
			// 更新推送状态为进行中【此处不使用批量更新，是考虑如果某个任务更新状态失败，无法识别】
			if (messageList != null && !messageList.isEmpty()) {
				log.info("MarketTaskPushService:found {} tasks", messageList.size());
				messageList.forEach(task -> {
					AdsMarketTaskInfo entity = new AdsMarketTaskInfo();
					entity.setTaskId(task.getTaskId());
					entity.setPushStatus(SmsPushStatus.doing.getValue());
					boolean ret = taskService.updateById(entity);
					if (ret) { // 更新成功，添加到推送队列
						pushService.asyncPushSms(task, 1);
					}
				});
			}
		} catch (Exception e) {
			log.info("CycleSendJob::findByMonth exception is {}", e.getMessage());
		}
	}

}
