package org.springblade.sms.kafka;

import java.util.Optional;
import java.util.Properties;
import java.util.concurrent.LinkedBlockingQueue;

import javax.annotation.PostConstruct;

import org.apache.commons.lang3.tuple.Triple;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.component.SmsService;
import org.springblade.sms.component.SmsStatusService;
import org.springblade.useranalysis.common.utils.Tuple;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;

import lombok.extern.slf4j.Slf4j;

/**
 * Kafka消费者
 *
 * @Author：william
 * @Date：2022/5/30
 */
@Slf4j
@Component
public class PushTaskConsumer {
	@Value("${spring.kafka.bootstrap-servers:}")
	private String servers;

	@Value("${spring.kafka.consumer.group-id:}")
	private String groupId;

	@Value("${spring.kafka.consumer.enable-auto-commit:}")
	private boolean autoCommit;

	@Value("${spring.kafka.consumer.auto-commit-interval:}")
	private int commitInterval;

	@Value("${spring.kafka.consumer.auto-offset-reset:}")
	private String autoOffsetReset;

	@Value("${spring.kafka.consumer.key-deserializer:}")
	private String keyDeserializer;

	@Value("${spring.kafka.consumer.value-deserializer:}")
	private String valueDeserializer;

	private KafkaConsumer<Object, Object> consumer;

	final static LinkedBlockingQueue<Tuple<String, String>> smsSpidQueue = new LinkedBlockingQueue<>(100);
	final static LinkedBlockingQueue<SmsDataKafka> smsQueue = new LinkedBlockingQueue<>(100);
	final static LinkedBlockingQueue<SmsStatusKafka> smsStatusQueue = new LinkedBlockingQueue<>(100);
	final static LinkedBlockingQueue<Triple<String, String, AdsMarketTaskInfo>> emailQueue = new LinkedBlockingQueue<>(
			100);

	@Autowired
	private SmsService smsService;

	@Autowired
	private SmsStatusService smsStatusService;

	@PostConstruct
	public void initKafkaConfig() {
		Properties props = new Properties();
		props.put("bootstrap.servers", servers);
		props.put("group.id", groupId);
		props.put("enable.auto.commit", autoCommit);
		props.put("auto.commit.interval.ms", commitInterval);
		props.put("auto-offset-reset", autoOffsetReset);
		props.put("key.deserializer", keyDeserializer);
		props.put("value.deserializer", valueDeserializer);
		consumer = new KafkaConsumer<>(props);
	}

	/**
	 * 产品促活短信发送消息监听
	 *
	 * @param record
	 */
	@KafkaListener(topics = { "active_spid" })
	public void onSpidSmsMessage(ConsumerRecord<String, Object> record) {
		Optional<?> optional = Optional.ofNullable(record.value());
		if (optional.isPresent()) {
			try {
				String mobile = record.key();
				String spid = optional.get().toString();
				log.info("PushTaskConsumer::receive spid sms from karfka,mobile is " + mobile);
				Tuple<String, String> tuple = new Tuple<>();
				tuple.setK(mobile);
				tuple.setV(spid);
				smsSpidQueue.put(tuple);
				consumer.commitSync();
				smsService.handleSmsSpidQueue(smsSpidQueue);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 短信发送消息监听
	 *
	 * @param record
	 */
	@KafkaListener(topics = { "sms" })
	public void onSmsMessage(ConsumerRecord<String, Object> record) {
		Optional<?> optional = Optional.ofNullable(record.value());
		if (optional.isPresent()) {
			try {
				Object json = optional.get();
				SmsDataKafka smsData = JSON.parseObject(json.toString(), SmsDataKafka.class);
				if (smsData != null) {
					log.info("PushTaskConsumer::receive sms task from karfka,mobile is " + smsData.getMobile());
				} else {
					log.info("PushTaskConsumer::receive sms task from karfka,but sms data is null");
				}
				smsQueue.put(smsData);
				smsService.handleSmsQueue(smsQueue);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			consumer.commitSync();
		}
	}

	/**
	 * 短信状态更新消息监听
	 *
	 * @param record
	 */
	@KafkaListener(topics = { "sms_status" })
	public void onSmsStatusMessage(ConsumerRecord<String, Object> record) {
		Optional<?> optional = Optional.ofNullable(record.value());
		if (optional.isPresent()) {
			try {
				Object json = optional.get();
				SmsStatusKafka smsStatusData = JSON.parseObject(json.toString(), SmsStatusKafka.class);
				smsStatusQueue.put(smsStatusData);
				smsStatusService.handle(smsStatusQueue);
			} catch (InterruptedException e) {
			}
			consumer.commitSync();
		}
	}

}
