package org.springblade.useranalysis.service.normal;

import java.util.List;
import java.util.stream.Collectors;

import org.springblade.useranalysis.dto.UserGroupRule;
import org.springblade.useranalysis.dto.UserGroupRuleDetail;
import org.springblade.useranalysis.dto.UserGroupSearchCondition;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.enums.UserGroupSearchFunction;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author william
 *	
 *	用户人群规则JSON解析
 */
@Slf4j
@Service
public class UserGroupRuleService {
	
	@Autowired
	private UserGroupService userGroupService;
	
	@Autowired
	private UserProfileService userProfileService;
	
	/**
	 * 根据人群ID解析
	 * @param groupId
	 * @return
	 */
	public QueryWrapper<ZUserTagEntity> parse(String groupId){
		return parse(userGroupService.getById(groupId));
	}
	
	/**
	 * 根据人群对象解析
	 * @param userGroup
	 * @return
	 */
	public QueryWrapper<ZUserTagEntity> parse(UserGroupEntity userGroup){
		if(userGroup == null) {
			return Wrappers.query();
		}
		if(userGroup.getRuleVersion() == 0) { //老版本
			return _oldParseRule(JSON.parseObject(userGroup.getRules()));
		}else { //新版本
			return _newParseRule(JSON.parseObject(userGroup.getRules(), UserGroupRule.class));
		}
	}
	
	/**
	 * 新版本规则解析
	 * @param ruleJson
	 * @return
	 */
	private QueryWrapper<ZUserTagEntity> _newParseRule(UserGroupRule userGroupRule){
		QueryWrapper<ZUserTagEntity> queryWrapper = Wrappers.query();
		if(userGroupRule == null || StrUtil.isEmpty(userGroupRule.getRelationship()) ||
				userGroupRule.getRuledetails() == null || userGroupRule.getRuledetails().isEmpty()) {
			return queryWrapper;
		}
		//1.解析规则
		List<UserGroupRuleDetail> ruledetails = userGroupRule.getRuledetails();
		String relationship = userGroupRule.getRelationship();
		if(ruledetails != null && !ruledetails.isEmpty()) {
			for(UserGroupRuleDetail ruleDetail : ruledetails) {
				if(ruleDetail.getType() == null) {
					throw new RuntimeException("userGroupRule｜无效的type");
				}
				//根据TagID查询标签对应clickhouse宽表数据类型和字段名【level2Name主要对未订购产品做处理(未订购和订购标签的tagId一样)】
				TagColumn whereCol = userProfileService.getColumNameByLevel2Name(ruleDetail.getLevel2Name());
				if(whereCol == null) {
					continue;
				}
				switch(ruleDetail.getType()) {
//					case string:{}
//					case linkup_area:{}
//					case linkup_channel:{}
//					case linkup:{
//						_buildQueryWrapper(relationship, whereCol, ruleDetail.getValues(), queryWrapper);
//						break;
//					}
					case date:{
						if(ruleDetail.getDate() == null || ruleDetail.getDate().size() < 2 
								|| StrUtil.isBlank(ruleDetail.getDate().get(0)) 
								|| StrUtil.isBlank(ruleDetail.getDate().get(1))) {
							_buildQueryWrapper(relationship, whereCol, ruleDetail.getValues(), queryWrapper);
						}else {
							_buildDateQueryWrapper(relationship, whereCol, ruleDetail, queryWrapper);
						}
						break;
					}
					case number:{
						List<UserGroupSearchCondition> conditions = ruleDetail.getConditions();
						if(conditions == null || conditions.isEmpty()) {
							_buildQueryWrapper(relationship, whereCol, ruleDetail.getValues(), queryWrapper);
						}else {
							_buildNumberQueryWrapper(relationship, whereCol, ruleDetail, queryWrapper);
						}
						break;
					}
					case string_search:{
						List<UserGroupSearchCondition> conditions = ruleDetail.getConditions();
						if(conditions == null || conditions.isEmpty()) {
							_buildQueryWrapper(relationship, whereCol, ruleDetail.getValues(), queryWrapper);
						}else {
							_buildStrSearchQueryWrapper(relationship, whereCol, ruleDetail, queryWrapper);
						}
						break;
					}
					default: {
						_buildQueryWrapper(relationship, whereCol, ruleDetail.getValues(), queryWrapper);
					}
				}
			}
		}
		//2.添加日期
		List<String> daterange = userGroupRule.getDaterange();
		if(daterange != null && daterange.size() == 2) {
			queryWrapper.ge("THEMONTH_V", daterange.get(0).replace("-", ""))
						.le("THEMONTH_V", daterange.get(1).replace("-", ""));
		}
		log.info("=======>>>>SQL:" + queryWrapper.getCustomSqlSegment());
		return queryWrapper;
	}
	
	/**
	 * 老版本规则解析
	 * @param ruleJson
	 * @return
	 */
	private QueryWrapper<ZUserTagEntity> _oldParseRule(JSONObject ruleJson){
		JSONArray ruleColums = ruleJson.getJSONArray("ruledetails");
		String relationship = ruleJson.getString("relationship");

		QueryWrapper<ZUserTagEntity> queryWrapper = Wrappers.query();
		for (int i = 0; i < ruleColums.size(); i++) {
			String key = ruleColums.getJSONObject(i).getString("key");
			String value = ruleColums.getJSONObject(i).getString("value");
			JSONArray jsonArr = JSONArray.parseArray(value);
			if (jsonArr != null && jsonArr.size() > 0) {
				// 根据标签code 获取列信息
				TagColumn whereCol = userProfileService.getColumName((String) jsonArr.get(0), key);
				if(whereCol == null) {
					continue;
				}
				_buildQueryWrapper(relationship, whereCol, jsonArr.toJavaList(String.class), queryWrapper);
			}
		}
		// 判断是否有日期
		JSONArray dateRangeArr = ruleJson.getJSONArray("daterange");
		if (dateRangeArr != null && dateRangeArr.size() == 2) {
			queryWrapper.ge("THEMONTH_V", dateRangeArr.getString(0).replace("-", ""))
						.le("THEMONTH_V", dateRangeArr.getString(1).replace("-", ""));
		}
		return queryWrapper;
	}
	
	/**
	 * 解析普通规则
	 * @param relationship
	 * @param tagCode
	 * @param tagName
	 * @param dataList
	 * @param queryWrapper
	 */
	private void _buildQueryWrapper(String relationship, TagColumn whereCol, List<String> dataList, QueryWrapper<ZUserTagEntity> queryWrapper) {
		if(dataList == null || dataList.isEmpty()) {
			return;
		}
		//数组使用hasAny(arr1, arr2)，非数组使用has(arr, element)
		//=0不包含，=1包含
		String arryValue = "[" + dataList.stream().map(s -> "\'" + s + "\'").collect(Collectors.joining(",")) + "]",
			   columnE = "event_unspname".equalsIgnoreCase(whereCol.englishName) ? "EVENT_SPNAME" : whereCol.columnE.toUpperCase(),
			   resultVal = "event_unspname".equalsIgnoreCase(whereCol.englishName) ? "0" : "1",
			   applySql = String.format("%s(%s, `%s`) = %s", (whereCol.columnType.equalsIgnoreCase("array") ? "hasAny" : "has"), arryValue, columnE, resultVal);
		// 且
		if (relationship.equals("and")) {
			queryWrapper.and(a -> a.apply(applySql));
		// 或
		} else {
			queryWrapper.or(a -> a.apply(applySql));
		}
	}
	/**
	 * 解析日期类型
	 * @param relationship
	 * @param whereCol
	 * @param ruleDetail
	 * @param queryWrapper
	 */
	private void _buildDateQueryWrapper(String relationship, TagColumn whereCol, UserGroupRuleDetail ruleDetail, QueryWrapper<ZUserTagEntity> queryWrapper) {
		List<String> times = ruleDetail.getDate();
		String startTime = times.get(0) + "-01", endTime1 = times.get(1) + "-01";
		//获取下个月的1号
		String endTime = DateUtil.parse(endTime1, "yyyy-MM-dd").offsetNew(DateField.MONTH, 1).toString("yyyy-MM-dd");
		if (relationship.equals("and")) {
			queryWrapper.and(a -> a.ge(whereCol.columnV.toUpperCase(), startTime).lt(whereCol.columnV.toUpperCase(), endTime));
		} else {
			queryWrapper.or(a -> a.ge(whereCol.columnV.toUpperCase(), startTime).lt(whereCol.columnV.toUpperCase(), endTime));
		}
	}
	/**
	 * 解析数字类型
	 * @param relationship
	 * @param whereCol
	 * @param ruleDetail
	 * @param queryWrapper
	 */
	private void _buildNumberQueryWrapper(String relationship, TagColumn whereCol, UserGroupRuleDetail ruleDetail, QueryWrapper<ZUserTagEntity> queryWrapper) {
		for(UserGroupSearchCondition condition : ruleDetail.getConditions()) {
			if(!NumberUtil.isNumber(condition.getVal())) { //如果不是数字，不做处理
				continue;
			}
			// 且
			if (relationship.equals("and")) {
				//arrayCount(item -> item > 10000, `EVENT_APP_USE.V`) > 1;
				if (whereCol.columnType.equalsIgnoreCase("array")) {
					queryWrapper.and(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), "arrayCount(item -> "+condition.getFunc().sql("item", condition.getVal())+", `"+whereCol.columnV.toUpperCase()+"`) > 0"));
				} else {
					queryWrapper.and(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), condition.getFunc().sql("`"+whereCol.columnV.toUpperCase()+"`", condition.getVal())));
				}
				// 或
			} else {
				if (whereCol.columnType.equalsIgnoreCase("array")) {
					queryWrapper.or(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), "arrayCount(item -> "+condition.getFunc().sql("item", condition.getVal())+", `"+whereCol.columnV.toUpperCase()+"`) > 0"));
				} else {
					queryWrapper.or(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), condition.getFunc().sql("`"+whereCol.columnV.toUpperCase()+"`", condition.getVal())));
				}
			}
		}
	}
	/**
	 * 字符串查询和匹配
	 * @param relationship
	 * @param whereCol
	 * @param ruleDetail
	 * @param queryWrapper
	 */
	private void _buildStrSearchQueryWrapper(String relationship, TagColumn whereCol, UserGroupRuleDetail ruleDetail, QueryWrapper<ZUserTagEntity> queryWrapper) {
		for(UserGroupSearchCondition condition : ruleDetail.getConditions()) {
			//字符串比较，只能精确匹配和模糊匹配
			if(condition.getFunc() != UserGroupSearchFunction.eq && condition.getFunc() != UserGroupSearchFunction.like) {
				continue;
			}
			// 且
			if (relationship.equals("and")) {
				//arrayExists(item -> item like '%10000%', `EVENT_AXXX.V`);
				if (whereCol.columnType.equalsIgnoreCase("array")) {
					queryWrapper.and(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), "arrayExists(item -> "+condition.getFunc().strSql("item", condition.getVal())+", `"+whereCol.columnV.toUpperCase()+"`)"));
				} else {
					queryWrapper.and(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), condition.getFunc().strSql("`"+whereCol.columnV.toUpperCase()+"`", condition.getVal())));
				}
				// 或
			} else {
				if (whereCol.columnType.equalsIgnoreCase("array")) {
					queryWrapper.or(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), "arrayExists(item -> "+condition.getFunc().strSql("item", condition.getVal())+", `"+whereCol.columnV.toUpperCase()+"`)"));
				} else {
					queryWrapper.or(a -> a.apply(StrUtil.isNotEmpty(condition.getVal()), condition.getFunc().strSql("`"+whereCol.columnV.toUpperCase()+"`", condition.getVal())));
				}
			}
		}
	}
}
